@@ -17,23 +17,51 @@ Copyright (C) 2010 Hiroki Ohtani(liris)
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""
+"""        
 
 
 import socket
 from urlparse import urlparse
-import random
+import os
 import struct
-import md5
+import uuid
+import sha
+import base64
 import logging
 
+"""
+websocket python client.
+=========================
+
+This version support only hybi-13.
+Please see http://tools.ietf.org/html/rfc6455 for protocol.
+"""
+
+
+# websocket supported version.
+VERSION = 13
+
+# closing frame status codes.
+STATUS_NORMAL = 1000
+STATUS_GOING_AWAY = 1001
+STATUS_PROTOCOL_ERROR = 1002
+STATUS_UNSUPPORTED_DATA_TYPE = 1003
+STATUS_STATUS_NOT_AVAILABLE = 1005
+STATUS_ABNORMAL_CLOSED = 1006
+STATUS_INVALID_PAYLOAD = 1007
+STATUS_POLICY_VIOLATION = 1008
+STATUS_MESSAGE_TOO_BIG = 1009
+STATUS_INVALID_EXTENSION = 1010
+STATUS_UNEXPECTED_CONDITION = 1011
+STATUS_TLS_HANDSHAKE_ERROR = 1015
+
 
 logger = logging.getLogger()
 
 class WebSocketException(Exception):
-    pass
-
-class ConnectionClosedException(WebSocketException):
+    """
+    websocket exeception class.
+    """
     pass
 
 default_timeout = None
@@ -42,6 +70,8 @@ traceEnabled = False
 def enableTrace(tracable):
     """
     turn on/off the tracability.
+
+    tracable: boolean value. if set True, tracability is enabled.
     """
     global traceEnabled
     traceEnabled = tracable
@@ -53,14 +83,15 @@ def enableTrace(tracable):
 def setdefaulttimeout(timeout):
     """
     Set the global timeout setting to connect.
+
+    timeout: default socket timeout time. This value is second.
     """
     global default_timeout
     default_timeout = timeout
 
-
 def getdefaulttimeout():
     """
-    Return the global timeout setting to connect.
+    Return the global timeout setting(second) to connect.
     """
     return default_timeout
 
@@ -68,8 +99,16 @@ def _parse_url(url):
     """
     parse url and the result is tuple of 
     (hostname, port, resource path and the flag of secure mode)
+
+    url: url string.
     """
-    parsed = urlparse(url)
+    if ":" not in url:
+        raise ValueError("url is invalid")
+
+    scheme, url = url.split(":", 1)
+    url = url.rstrip("/")
+
+    parsed = urlparse(url, scheme="http")
     if parsed.hostname:
         hostname = parsed.hostname
     else:
@@ -77,25 +116,24 @@ def _parse_url(url):
     port = 0
     if parsed.port:
         port = parsed.port
-   
+
     is_secure = False
-    if parsed.scheme == "ws":
+    if scheme == "ws":
         if not port:
             port = 80
-    elif parsed.scheme == "wss":
+    elif scheme == "wss":
         is_secure = True
         if not port:
             port  = 443
     else:
-        raise ValueError("scheme %s is invalid" % parsed.scheme)
-    
+        raise ValueError("scheme %s is invalid" % scheme)
+
     if parsed.path:
         resource = parsed.path
     else:
         resource = "/"
-    
-    return (hostname, port, resource, is_secure)
 
+    return (hostname, port, resource, is_secure)
 
 def create_connection(url, timeout=None, **options):
     """
@@ -104,6 +142,17 @@ def create_connection(url, timeout=None, **options):
     Connect to url and return the WebSocket object.
     Passing optional timeout parameter will set the timeout on the socket.
     If no timeout is supplied, the global default timeout setting returned by getdefauttimeout() is used.
+    You can customize using 'options'.
+    If you set "headers" dict object, you can set your own custom header.
+
+    >>> conn = create_connection("ws://echo.websocket.org/",
+    ...     headers={"User-Agent": "MyProgram"})
+
+    timeout: socket timeout time. This value is integer.
+             if you set None for this value, it means "use default_timeout value"
+
+    options: current support option is only "header".
+             if you set header as dict value, the custom HTTP headers are added.
     """
     websock = WebSocket()
     websock.settimeout(timeout != None and timeout or default_timeout)
@@ -118,39 +167,14 @@ _MAX_CHAR_BYTE = (1<<8) -1
 # http://axod.blogspot.com/2010/06/websocket-gets-update-and-it-breaks.html
 
 def _create_sec_websocket_key():
-    spaces_n = random.randint(1, 12)
-    max_n = _MAX_INTEGER / spaces_n
-    number_n = random.randint(0, max_n)
-    product_n = number_n * spaces_n
-    key_n = str(product_n)
-    for i in range(random.randint(1, 12)):
-        c = random.choice(_AVAILABLE_KEY_CHARS)
-        pos = random.randint(0, len(key_n))
-        key_n = key_n[0:pos] + chr(c) + key_n[pos:]
-    for i in range(spaces_n):
-        pos = random.randint(1, len(key_n)-1)
-        key_n = key_n[0:pos] + " " + key_n[pos:]
-    
-    return number_n, key_n
+    uid = uuid.uuid1()
+    return base64.encodestring(uid.bytes).strip()
 
-def _create_key3():
-    return "".join([chr(random.randint(0, _MAX_CHAR_BYTE)) for i in range(8)])
-
-HEADERS_TO_CHECK = {
+_HEADERS_TO_CHECK = {
     "upgrade": "websocket",
     "connection": "upgrade",
     }
 
-HEADERS_TO_EXIST_FOR_HYBI00 = [
-    "sec-websocket-origin",
-    "sec-websocket-location",
-]
-
-HEADERS_TO_EXIST_FOR_HIXIE75 = [
-    "websocket-origin",
-    "websocket-location",
-]
-
 class _SSLSocketWrapper(object):
     def __init__(self, sock):
         self.ssl = socket.ssl(sock)
@@ -161,6 +185,118 @@ class _SSLSocketWrapper(object):
     def send(self, payload):
         return self.ssl.write(payload)
 
+_BOOL_VALUES = (0, 1)
+def _is_bool(*values):
+    for v in values:
+        if v not in _BOOL_VALUES:
+            return False
+    
+    return True
+
+class ABNF(object):
+    """
+    ABNF frame class.
+    see http://tools.ietf.org/html/rfc5234
+    and http://tools.ietf.org/html/rfc6455#section-5.2
+    """
+    
+    # operation code values.
+    OPCODE_TEXT   = 0x1
+    OPCODE_BINARY = 0x2
+    OPCODE_CLOSE  = 0x8
+    OPCODE_PING   = 0x9
+    OPCODE_PONG   = 0xa
+    
+    # available operation code value tuple
+    OPCODES = (OPCODE_TEXT, OPCODE_BINARY, OPCODE_CLOSE,
+                OPCODE_PING, OPCODE_PONG)
+
+    # data length threashold.
+    LENGTH_7  = 0x7d
+    LENGTH_16 = 1 << 16
+    LENGTH_63 = 1 << 63
+
+    def __init__(self, fin = 0, rsv1 = 0, rsv2 = 0, rsv3 = 0,
+                 opcode = OPCODE_TEXT, mask = 1, data = ""):
+        """
+        Constructor for ABNF.
+        please check RFC for arguments.
+        """
+        self.fin = fin
+        self.rsv1 = rsv1
+        self.rsv2 = rsv2
+        self.rsv3 = rsv3
+        self.opcode = opcode
+        self.mask = mask
+        self.data = data
+        self.get_mask_key = os.urandom
+
+    @staticmethod
+    def create_frame(data, opcode):
+        """
+        create frame to send text, binary and other data.
+        
+        data: data to send. This is string value(byte array).
+            if opcode is OPCODE_TEXT and this value is uniocde,
+            data value is conveted into unicode string, automatically.
+
+        opcode: operation code. please see OPCODE_XXX.
+        """
+        if opcode == ABNF.OPCODE_TEXT and isinstance(data, unicode):
+            data = data.encode("utf-8")
+        # mask must be set if send data from client
+        return ABNF(1, 0, 0, 0, opcode, 1, data)
+
+    def format(self):
+        """
+        format this object to string(byte array) to send data to server.
+        """
+        if not _is_bool(self.fin, self.rsv1, self.rsv2, self.rsv3):
+            raise ValueError("not 0 or 1")
+        if self.opcode not in ABNF.OPCODES:
+            raise ValueError("Invalid OPCODE")
+        length = len(self.data)
+        if length >= ABNF.LENGTH_63:
+            raise ValueError("data is too long")
+        
+        frame_header = chr(self.fin << 7
+                           | self.rsv1 << 6 | self.rsv2 << 5 | self.rsv3 << 4
+                           | self.opcode)
+        if length < ABNF.LENGTH_7:
+            frame_header += chr(self.mask << 7 | length)
+        elif length <= ABNF.LENGTH_16:
+            frame_header += chr(self.mask << 7 | 0x7e)
+            frame_header += struct.pack("!H", length)
+        else:
+            frame_header += chr(self.mask << 7 | 0x7f)
+            frame_header += struct.pack("!Q", length)
+        
+        if not self.mask:
+            return frame_header + self.data
+        else:
+            mask_key = self.get_mask_key(4)
+            return frame_header + self._get_masked(mask_key)
+
+    def _get_masked(self, mask_key):
+        s = ABNF.mask(mask_key, self.data)
+        return mask_key + "".join(s)
+
+    @staticmethod
+    def mask(mask_key, data):
+        """
+        mask or unmask data. Just do xor for each byte
+
+        mask_key: 4 byte string(byte).
+        
+        data: data to mask/unmask.
+        """
+        _m = map(ord, mask_key)
+        _d = map(ord, data)
+        for i in range(len(_d)):
+            _d[i] ^= _m[i % 4]
+        s = map(chr, _d)
+        return "".join(s)
+
 class WebSocket(object):
     """
     Low level WebSocket interface.
@@ -173,7 +309,7 @@ class WebSocket(object):
 
     >>> import websocket
     >>> ws = websocket.WebSocket()
-    >>> ws.Connect("ws://localhost:8080/echo")
+    >>> ws.connect("ws://echo.websocket.org")
     >>> ws.send("Hello, Server")
     >>> ws.recv()
     'Hello, Server'
@@ -185,22 +321,52 @@ class WebSocket(object):
         """
         self.connected = False
         self.io_sock = self.sock = socket.socket()
+        self.get_mask_key = None
+        
+    def set_mask_key(self, func):
+        """
+        set function to create musk key. You can custumize mask key generator.
+        Mainly, this is for testing purpose.
+
+        func: callable object. the fuct must 1 argument as integer.
+              The argument means length of mask key.
+              This func must be return string(byte array),
+              which length is argument specified.
+        """
+        self.get_mask_key = func
 
     def settimeout(self, timeout):
         """
         Set the timeout to the websocket.
+        
+        timeout: timeout time(second).
         """
         self.sock.settimeout(timeout)
 
     def gettimeout(self):
         """
-        Get the websocket timeout.
+        Get the websocket timeout(second).
         """
         return self.sock.gettimeout()
     
     def connect(self, url, **options):
         """
         Connect to url. url is websocket url scheme. ie. ws://host:port/resource
+        You can customize using 'options'.
+        If you set "headers" dict object, you can set your own custom header.
+        
+        >>> ws = WebSocket()
+        >>> ws.connect("ws://echo.websocket.org/",
+        ...     headers={"User-Agent": "MyProgram"})
+
+        timeout: socket timeout time. This value is integer.
+                 if you set None for this value,
+                 it means "use default_timeout value"
+
+        options: current support option is only "header".
+                 if you set header as dict value,
+                 the custom HTTP headers are added.
+
         """
         hostname, port, resource, is_secure = _parse_url(url)
         # TODO: we need to support proxy
@@ -212,11 +378,8 @@ class WebSocket(object):
     def _handshake(self, host, port, resource, **options):
         sock = self.io_sock
         headers = []
-        if "header" in options:
-            headers.extend(options["header"])
-
         headers.append("GET %s HTTP/1.1" % resource)
-        headers.append("Upgrade: WebSocket")
+        headers.append("Upgrade: websocket")
         headers.append("Connection: Upgrade")
         if port == 80:
             hostport = host
@@ -225,13 +388,15 @@ class WebSocket(object):
         headers.append("Host: %s" % hostport)
         headers.append("Origin: %s" % hostport)
    
-        number_1, key_1 = _create_sec_websocket_key()
-        headers.append("Sec-WebSocket-Key1: %s" % key_1)
-        number_2, key_2 = _create_sec_websocket_key()
-        headers.append("Sec-WebSocket-Key2: %s" % key_2)
+        key = _create_sec_websocket_key()
+        headers.append("Sec-WebSocket-Key: %s" % key)
+        headers.append("Sec-WebSocket-Protocol: chat, superchat")
+        headers.append("Sec-WebSocket-Version: %s" % VERSION)
+        if "header" in options:
+            headers.extend(options["header"])
+
+        headers.append("")
         headers.append("")
-        key3 = _create_key3()
-        headers.append(key3)
 
         header_str = "\r\n".join(headers)
         sock.send(header_str)
@@ -244,61 +409,31 @@ class WebSocket(object):
         if status != 101:
             self.close()
             raise WebSocketException("Handshake Status %d" % status)
-        success, secure = self._validate_header(resp_headers)
+
+        success = self._validate_header(resp_headers, key)
         if not success:
             self.close()
             raise WebSocketException("Invalid WebSocket Header")
-        
-        if secure:
-            resp = self._get_resp()
-            if not self._validate_resp(number_1, number_2, key3, resp):
-                self.close()
-                raise WebSocketException("challenge-response error")
 
         self.connected = True
     
-    def _validate_resp(self, number_1, number_2, key3, resp):
-        challenge = struct.pack("!I", number_1)
-        challenge += struct.pack("!I", number_2)
-        challenge += key3
-        digest = md5.md5(challenge).digest()
+    def _validate_header(self, headers, key):
+        for k, v in _HEADERS_TO_CHECK.iteritems():
+            r = headers.get(k, None)
+            if not r:
+                return False
+            r = r.lower()
+            if v != r:
+                return False
+
+        result = headers.get("sec-websocket-accept", None)
+        if not result:
+            return False
+        result = result.lower()
         
-        return  resp == digest
-
-    def _get_resp(self):
-        result = self._recv(16)
-        if traceEnabled:
-            logger.debug("--- challenge response result ---")
-            logger.debug(repr(result))
-            logger.debug("---------------------------------")
-        
-        return result
-
-    def _validate_header(self, headers):
-        #TODO: check other headers
-        for key, value in HEADERS_TO_CHECK.iteritems():
-            v = headers.get(key, None)
-            if value != v:
-                return False, False
-
-        success = 0
-        for key in HEADERS_TO_EXIST_FOR_HYBI00:
-            if key in headers:
-                success += 1
-        if success == len(HEADERS_TO_EXIST_FOR_HYBI00):
-            return True, True
-        elif success != 0:
-            return False, True
-
-        success = 0
-        for key in HEADERS_TO_EXIST_FOR_HIXIE75:
-            if key in headers:
-                success += 1
-        if success == len(HEADERS_TO_EXIST_FOR_HIXIE75):
-            return True, False
-
-        return False, False
-            
+        value = key + "258EAFA5-E914-47DA-95CA-C5AB0DC85B11"
+        hashed = base64.encodestring(sha.sha(value).digest()).strip().lower()
+        return hashed == result
 
     def _read_headers(self):
         status = None
@@ -329,69 +464,137 @@ class WebSocket(object):
         
         return status, headers    
     
-    def send(self, payload):
+    def send(self, payload, opcode = ABNF.OPCODE_TEXT):
         """
-        Send the data as string. payload must be utf-8 string or unicoce.
+        Send the data as string. 
+
+        payload: Payload must be utf-8 string or unicoce,
+                  if the opcode is OPCODE_TEXT.
+                  Otherwise, it must be string(byte array)
+
+        opcode: operation code to send. Please see OPCODE_XXX.
         """
-        if isinstance(payload, unicode):
-            payload = payload.encode("utf-8")
-        data = "".join(["\x00", payload, "\xff"])
+        frame = ABNF.create_frame(payload, opcode)
+        if self.get_mask_key:
+            frame.get_mask_key = self.get_mask_key
+        data = frame.format()
         self.io_sock.send(data)
         if traceEnabled:
             logger.debug("send: " + repr(data))
 
+    def ping(self, payload = ""):
+        """
+        send ping data.
+        
+        payload: data payload to send server.
+        """
+        self.send(payload, ABNF.OPCODE_PING)
+
+    def pong(self, payload):
+        """
+        send pong data.
+        
+        payload: data payload to send server.
+        """
+        self.send(payload, ABNF.OPCODE_PONG)
+
     def recv(self):
         """
-        Reeive utf-8 string data from the server.
+        Receive string data(byte array) from the server.
+
+        return value: string(byte array) value.
         """
-        b = self._recv(1)
-        if enableTrace:
-            logger.debug("recv frame: " + repr(b))
-        frame_type = ord(b)
-        if frame_type == 0x00:
-            bytes = []
-            while True:
-                b = self._recv(1)
-                if b == "\xff":
-                    break
-                else:
-                    bytes.append(b)
-            return "".join(bytes)
-        elif 0x80 < frame_type < 0xff:
-            # which frame type is valid?
-            length = self._read_length()
-            bytes = self._recv_strict(length)
-            return bytes
-        elif frame_type == 0xff:
-            n = self._recv(1)
-            self._closeInternal()
-            return None
-        else:
-            raise WebSocketException("Invalid frame type")
+        opcode, data = self.recv_data()
+        return data
 
-    def _read_length(self):
-        length = 0
+    def recv_data(self):
+        """
+        Recieve data with operation code.
+        
+        return  value: tuple of operation code and string(byte array) value.
+        """
         while True:
-            b = ord(self._recv(1))
-            length = length * (1 << 7) + (b & 0x7f)
-            if b < 0x80:
-                break
-            
-        return length
+            frame = self.recv_frame()
+            if frame.opcode in (ABNF.OPCODE_TEXT, ABNF.OPCODE_BINARY):
+                return (frame.opcode, frame.data)
+            elif frame.opcode == ABNF.OPCODE_CLOSE:
+                self.send_close()
+                return (frame.opcode, None)
+            elif frame.opcode == ABNF.OPCODE_PING:
+                self.pong("Hi!")
 
-    def close(self):
+
+    def recv_frame(self):
+        """
+        recieve data as frame from server.
+
+        return value: ABNF frame object.
+        """
+        header_bytes = self._recv(2)
+        if not header_bytes:
+            return None
+        b1 = ord(header_bytes[0])
+        fin = b1 >> 7 & 1
+        rsv1 = b1 >> 6 & 1
+        rsv2 = b1 >> 5 & 1
+        rsv3 = b1 >> 4 & 1
+        opcode = b1 & 0xf
+        
+        b2 = ord(header_bytes[1])
+        mask = b2 >> 7 & 1
+        length = b2 & 0x7f
+
+        if length == 0x7e:
+            l = self._recv(2)
+            length = struct.unpack("!H", l)[0]
+        elif length == 0x7f:
+            l = self._recv(8)
+            length = struct.unpack("!Q", l)[0]
+
+        if mask:
+            mask_key = self._recv(4)
+        data = self._recv(length)
+        if mask:
+            data = ABNF.mask(mask_key, data)
+        
+        frame = ABNF(fin, rsv1, rsv2, rsv3, opcode, mask, data)
+        return frame
+
+    def send_close(self, status = STATUS_NORMAL, reason = ""):
+        """
+        send close data to the server.
+        
+        status: status code to send. see STATUS_XXX.
+
+        reason: the reason to close. This must be string.
+        """
+        if status < 0 or status > ABNF.LENGTH_16:
+            raise ValueError("code is invalid range")
+        self.send(struct.pack('!H', status) + reason, ABNF.OPCODE_CLOSE)
+        
+
+
+    def close(self, status = STATUS_NORMAL, reason = ""):
         """
         Close Websocket object
+
+        status: status code to send. see STATUS_XXX.
+
+        reason: the reason to close. This must be string.
         """
         if self.connected:
+            if status < 0 or status > ABNF.LENGTH_16:
+                raise ValueError("code is invalid range")
+
             try:
-                self.io_sock.send("\xff\x00")
+                self.send(struct.pack('!H', status) + reason, ABNF.OPCODE_CLOSE)
                 timeout = self.sock.gettimeout()
-                self.sock.settimeout(1)
+                self.sock.settimeout(3)
                 try:
-                    result = self._recv(2)
-                    if result != "\xff\x00":
-                        logger.error("bad closing Handshake")
+                    frame = self.recv_frame()
+                    print repr(frame.data)
+                    if logger.isEnabledFor(logging.DEBUG):
+                        logger.error("close status: " + repr(frame.data))
                 except:
                     pass
                 self.sock.settimeout(timeout)
@@ -407,8 +610,6 @@ class WebSocket(object):
         
     def _recv(self, bufsize):
         bytes = self.io_sock.recv(bufsize)
-        if not bytes:
-            raise ConnectionClosedException()
         return bytes
 
     def _recv_strict(self, bufsize):
@@ -505,8 +706,7 @@ class WebSocketApp(object):
 
 if __name__ == "__main__":
     enableTrace(True)
-    #ws = create_connection("ws://localhost:8080/echo")
-    ws = create_connection("ws://localhost:5000/chat")
+    ws = create_connection("ws://echo.websocket.org/")
     print "Sending 'Hello, World'..."
     ws.send("Hello, World")
     print "Sent"
@@ -514,8 +714,3 @@ if __name__ == "__main__":
     result =  ws.recv()
     print "Received '%s'" % result
     ws.close()
-        
-
-
-
-
